@@ -1,7 +1,7 @@
 /*
  * gauche-gtk.h - Gauche+Gtk extension
  *
- *  Copyright(C) 2002 by Shiro Kawai (shiro@acm.org)
+ *  Copyright(C) 2002-2004 by Shiro Kawai (shiro@acm.org)
  *
  *  Permission to use, copy, modify, distribute this software and
  *  accompanying documentation for any purpose is hereby granted,
@@ -12,7 +12,7 @@
  *  warranty.  In no circumstances the author(s) shall be liable
  *  for any damages arising out of the use of this software.
  *
- *  $Id: gauche-gtk.c,v 1.46 2003/12/09 20:16:03 shirok Exp $
+ *  $Id: gauche-gtk.c,v 1.51 2004/03/18 21:21:37 shirok Exp $
  */
 
 #include "gauche-gtk.h"
@@ -490,7 +490,7 @@ ScmObj Scm_UnboxGValue(const GValue *gv)
     case G_TYPE_UCHAR: return SCM_MAKE_INT((int)g_value_get_uchar(gv));
     case G_TYPE_BOOLEAN: return SCM_MAKE_BOOL(g_value_get_boolean(gv));
     case G_TYPE_INT:   return Scm_MakeInteger(g_value_get_int(gv));
-    case G_TYPE_UINT:  return Scm_MakeIntegerFromUI(g_value_get_uint(gv));
+    case G_TYPE_UINT:  return Scm_MakeIntegerU(g_value_get_uint(gv));
     case G_TYPE_LONG:  return Scm_MakeInteger(g_value_get_long(gv));
     case G_TYPE_ULONG: return Scm_MakeInteger(g_value_get_ulong(gv));
     case G_TYPE_FLOAT: return Scm_MakeFlonum((double)g_value_get_float(gv));
@@ -511,9 +511,6 @@ ScmObj Scm_UnboxGValue(const GValue *gv)
         if (g_type_is_a (gt, G_TYPE_FLAGS)) {
             return Scm_MakeInteger(g_value_get_flags(gv));
         }
-        if (gt == GDK_TYPE_EVENT) {
-            return Scm_MakeGdkEvent((GdkEvent*)g_value_get_boxed(gv));
-        }
         if (gt == gtk_tree_path_get_type()) {
             return SCM_MAKE_GTK_TREE_PATH((GtkTreePath*)g_value_get_boxed(gv));
         }
@@ -560,7 +557,7 @@ void Scm_BoxGValue(GValue *gv, ScmObj sv)
     }
     case G_TYPE_UINT: {
         if (!SCM_EXACTP(sv)) goto err;
-        g_value_set_uint(gv, Scm_GetUInteger(sv));
+        g_value_set_uint(gv, Scm_GetIntegerU(sv));
         return;
     }
     case G_TYPE_LONG: {
@@ -570,7 +567,7 @@ void Scm_BoxGValue(GValue *gv, ScmObj sv)
     }
     case G_TYPE_ULONG: {
         if (!SCM_EXACTP(sv)) goto err;
-        g_value_set_ulong(gv, Scm_GetUInteger(sv));
+        g_value_set_ulong(gv, Scm_GetIntegerU(sv));
         return;
     }
     case G_TYPE_FLOAT: {
@@ -787,6 +784,28 @@ ScmObj Scm_MakeGTimer(GTimer *r)
     return SCM_OBJ(g);
 }
 
+/*===============================================================
+ * Pango auxiliary structures
+ */
+
+SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_PangoLayoutIterClass, NULL);
+
+static void pango_layout_iter_finalize(ScmObj obj, void *data)
+{
+    ScmPangoLayoutIter *g = (ScmPangoLayoutIter*)obj;
+    pango_layout_iter_free(g->iter);
+    g->iter = NULL;
+}
+
+ScmObj Scm_MakePangoLayoutIter(PangoLayoutIter *r)
+{
+    ScmPangoLayoutIter *g = SCM_NEW(ScmPangoLayoutIter);
+    SCM_SET_CLASS(g, SCM_CLASS_PANGO_LAYOUT_ITER);
+    g->iter = r;
+    Scm_RegisterFinalizer(SCM_OBJ(g), pango_layout_iter_finalize, NULL);
+    return SCM_OBJ(g);
+}
+
 /*===============================================================
  * GdkAtom <-> ScmObj mapping
  */
@@ -840,6 +859,49 @@ ScmObj Scm_GtkGcharPtrBox(gchar *string)
     return s;
 }
 
+/*===============================================================
+ * uvector <-> property data (used by gtk-property stuff)
+ */
+
+/* gtk-property-change accepts u8, u16, and u32vector as the
+   property data.  u8 and u16 vectors are directly mapped to
+   guchar* and gushort*.  We need a special care to map u32vector
+   to gulong*. */
+guchar* Scm_GdkPropertyDataFromUVector(ScmObj uvec,
+                                       int *format,    /* out */
+                                       int *nelements) /* out */
+{
+    if (SCM_U8VECTORP(uvec)) {
+        *nelements = SCM_U8VECTOR_SIZE(uvec);
+        *format = 8;
+        return (guchar*)SCM_U8VECTOR_ELEMENTS(uvec);
+    } else if (SCM_U16VECTORP(uvec)) {
+        *nelements = SCM_U16VECTOR_SIZE(uvec);
+        *format = 16;
+        return (guchar*)SCM_U16VECTOR_ELEMENTS(uvec);
+    } else if (SCM_U32VECTORP(uvec)) {
+        *nelements = SCM_U32VECTOR_SIZE(uvec);
+        *format = 32;
+#if SIZEOF_LONG == 4
+        return (guchar*)SCM_U32VECTOR_ELEMENTS(uvec);
+#else
+        {
+            gulong *buf = SCM_NEW_ATOMIC2(gulong*,
+                                          (*nelements)*sizeof(gulong));
+            int i;
+            for (i=0; i<*nelements; i++) {
+                buf[i] = (gulong)(SCM_U32VECTOR_ELEMENTS(uvec)[i]);
+            }
+            return (guchar*)buf;
+        }
+#endif
+    } else {
+        Scm_Error("property data must be either u8, u16, or u32vector, but got %S", uvec);
+        return NULL;            /* dummy */
+    }
+}
+
+
 /*===============================================================
  * GdkEvent
  */
@@ -1202,6 +1264,9 @@ void Scm_Init_gauche_gtk(void)
     Scm_InitBuiltinClass(&Scm_GTypeClass, "<g-type>",
                          NULL, sizeof(ScmGType),
                          mod);
+    Scm_InitBuiltinClass(&Scm_PangoLayoutIterClass, "<pango-layout-iter>",
+                         NULL, sizeof(ScmPangoLayoutIter),
+                         mod);
     Scm_InitBuiltinClass(&Scm_GdkAtomClass, "<gdk-atom>",
                          NULL, sizeof(ScmGdkAtom),
                          mod);
