@@ -1,6 +1,6 @@
 ;;
 ;; parse gtk headers to generate stub file
-;; $Id: h2stub.scm,v 1.42 2003/01/15 06:55:24 shirok Exp $
+;; $Id: h2stub.scm,v 1.51 2004/06/24 11:49:19 shirok Exp $
 
 ;; OK.  Defs file should be the way to go.  However, gtk+ distribution
 ;; doesn't include defs file, and I couldn't find one for gtk-2.0 in
@@ -39,16 +39,15 @@
 ;; We first try pkg-config.  If it fails, does
 ;; heuristic search.  
 (define *header-search-paths*
-  (map (cut build-path <> "include")
-       (delete-duplicates
-        (append (call-with-input-process
-                 #`"pkg-config --variable=prefix gtk+-,(gtk-version)"
-                 (cut port->string-list <>))
-                (call-with-input-process
-                 #`"pkg-config --variable=prefix pango-,(pango-version)"
-                 (cut port->string-list <>))
-                '("/usr" "/usr/local"))
-        string=?)))
+  (delete-duplicates
+   (append (call-with-input-process
+               #`"pkg-config --variable=includedir gtk+-,(gtk-version)"
+             (cut port->string-list <>))
+           (call-with-input-process
+               #`"pkg-config --variable=includedir pango-,(pango-version)"
+             (cut port->string-list <>))
+           '("/usr/include" "/usr/local/include"))
+   string=?))
 
 (define (find-header-dir target paths)
   (cond ((find-file-in-paths target
@@ -173,6 +172,7 @@
    (guint16    <uint16>)
    (guint32    <uint32>)
    (gulong     <ulong>)
+   (guchar     <uint8>)
    (gushort    <uint16>)
    (gboolean   <boolean>)
    (gfloat     <float>)
@@ -191,13 +191,18 @@
    ;; from ScmGetStringConst*.
    (const-char*  <const-char*>)
    (const-gchar* <const-gchar*>)
+   ;; Generic GObject
+   (GObject* <g-object>)
    ;; This is used to box the returned allocated gchar*
    (gchar*       <gchar*>)
    ;; Opaque types
    (PangoContext* <pango-context>)
    (PangoLanguage* <pango-language>)
+   (PangoAttrList* <pango-attr-list>)
+   (PangoLayoutIter* <pango-layout-iter>)
    (GdkAtom    <gdk-atom>)
    (GdkRegion* <gdk-region>)
+   (GdkPixbufFormat* <gdk-pixbuf-format>)
    (GtkTreePath* <gtk-tree-path>)
    (GtkTreeRowReference* <gtk-tree-row-reference>)
    ;; GdkEvent is a union.
@@ -220,8 +225,8 @@
    ;; the following slots are used by field info
    (read-only? :initform #f :accessor read-only?)
    (accessible? :initform #t :accessor accessible?)
-   (getter     :initform #f :accessor getter-of)
-   (setter     :initform #f :accessor setter-of)
+   (getter     :init-keyword :getter :initform #f :accessor getter-of)
+   (setter     :init-keyword :setter :initform #f :accessor setter-of)
    ))
 
 (define-method write-object ((self <gtk-var>) port)
@@ -455,7 +460,7 @@
 
 (define (parse-header filename)
   (parameterize ((input-file filename))
-    (report #`"parsing ,(sys-basename filename)")
+    (report #`"parsing ,filename")
     (with-input-from-file filename parse-body)))
 
 (define (parse-body)
@@ -563,10 +568,10 @@
       (test eof-object? (err-eof))
       (#/^\s*$/ () (loop (read-line) args))
       (test has-comment? (skip-comment line (cut loop <> args) err-eof))
+      (#/\s*([^,]+),(.*)/ (#f arg rest)
+        (loop rest (cons arg args)))
       (#/\s*(.+)\).*\;/ (#f arg)
         (finish-function ret name (cons arg args)))
-      (#/\s*([^,]+),/ (#f arg)
-        (loop (read-line) (cons arg args)))
       (else
        (warn "~s in ~a" line name)
        (loop (read-line) args)))))
@@ -621,6 +626,14 @@
   `(cond ((find-function ',name)
           => (lambda (f) (set! (internal? f) #t)))))
 
+(define-macro (fix-arguments! args)
+  `(set! (arguments-of self) ,args))
+
+(define-macro (fix-body! body)
+  `(begin
+     (set! (return-type-of self) #f)
+     (set! (body-of self) ,body)))
+
 (define-macro (define-cclass-fix name . body)
   `(cclass-fix ',name (lambda (self) ,@body)))
 
@@ -636,7 +649,7 @@
 
 (define-macro (fix-field! name . body)
   `((lambda (field) . ,body)
-    (or (find (lambda (p) (eq? (scm-name-of p) ',name))
+    (or (find (lambda (p) (eq? (scm-name-of p) ,name))
               (fields-of self))
         (error "no such field" ',name))))
 
@@ -648,6 +661,19 @@
                    :type (find-type-or-create ',typesig)
                    ,@opts))))
 
+;; some field may be missing in certain gtk versions.
+;; use this to ignore such fields.  NB: there may be more than
+;; one field registered, if such fields appears within union.
+(define-macro (ignore-field! name)
+  `(for-each (lambda (f) (set! (accessible? f) #f))
+             (filter (lambda (p) (eq? (scm-name-of p) ,name))
+                     (fields-of self))))
+
+(define-macro (ignore-field-except! names)
+  `(for-each (lambda (f) (set! (accessible? f) #f))
+             (remove (lambda (p) (memq (scm-name-of p) ,names))
+                     (fields-of self))))
+
 (define-macro (add-mixin! . c-mixin-names)
   `(begin (set! (direct-supers-of self)
                 (list ,@c-mixin-names (car (cpl-of self))))
@@ -1248,6 +1274,10 @@
   (parse-headers #`",(gtk-directory)/gdk"
                  (call-with-input-file "GDKFILES" port->string-list)))
 
+(define (parse-gdk-pixbuf)
+  (parse-headers #`",(gtk-directory)/gdk-pixbuf"
+                 (call-with-input-file "GDKPIXBUFFILES" port->string-list)))
+
 (define (parse-gtk)
   (parse-headers #`",(gtk-directory)/gtk"
                  (call-with-input-file "GTKFILES" port->string-list)))
@@ -1260,6 +1290,7 @@
   (report "Parsing ...")
   (parse-pango)
   (parse-gdk)
+  (parse-gdk-pixbuf)
   (parse-gtk)
   (report "Fixing up ...")
   (fixup)
